@@ -16,13 +16,13 @@ from sqlalchemy import (Column, Integer, DateTime, JSON, ForeignKey, Boolean,
 from sqlalchemy.orm import relationship, deferred, Session, reconstructor
 from sqlalchemy.exc import IntegrityError, OperationalError
 
-from hpcflow import CONFIG, FILE_NAMES
+from hpcflow.config import Config as CONFIG
 from hpcflow._version import __version__
 from hpcflow.archive.archive import Archive, TaskArchiveStatus
 from hpcflow.base_db import Base
 from hpcflow.archive.cloud.cloud import CloudProvider
 from hpcflow.nesting import NestingType
-from hpcflow.scheduler import SunGridEngine, DirectExecution
+from hpcflow.scheduler import SunGridEngine
 from hpcflow.utils import coerce_same_length, zeropad, format_time_delta, get_random_hex, datetime_to_dict, timedelta_to_dict
 from hpcflow.validation import validate_task_multiplicity
 from hpcflow.variables import (
@@ -32,7 +32,6 @@ from hpcflow.variables import (
 
 SCHEDULER_MAP = {
     'sge': SunGridEngine,
-    'direct': DirectExecution,
 }
 
 
@@ -57,6 +56,7 @@ class Workflow(Base):
     root_archive_directory = Column(String(255), nullable=True)
     _profile_files = Column('profile_files', JSON, nullable=True)
     loop = Column(JSON)
+    parallel_modes = Column(JSON, nullable=True)
 
     command_groups = relationship(
         'CommandGroup',
@@ -71,7 +71,8 @@ class Workflow(Base):
 
     def __init__(self, directory, command_groups, var_definitions=None,
                  pre_commands=None, archives=None, root_archive_idx=None,
-                 root_archive_excludes=None, profile_files=None, loop=None):
+                 root_archive_excludes=None, profile_files=None, loop=None,
+                 parallel_modes=None):
         """Method to initialise a new Workflow.
 
         Parameters
@@ -110,7 +111,14 @@ class Workflow(Base):
                 groups : list of int, optional
                     Which command groups to include in iterations beyond the first. If not
                     specified, all command groups are included in the loop.
-
+        parallel_modes : dict, optional
+            If specified, (case-insensitive) keys are one or more of: 'MPI', 'OpenMP'.
+            Each is a dict with allowed keys:
+                env : list of str
+                    Environment set up required for a given parallel mode.
+                command : str
+                    Command to prepend to any command group commands that use this
+                    parallel mode.
 
         """
 
@@ -127,7 +135,7 @@ class Workflow(Base):
             if 'directory' in i:
 
                 var_names = extract_variable_names(
-                    i['directory'], CONFIG['variable_delimiters'])
+                    i['directory'], CONFIG.get('variable_delimiters'))
                 if len(var_names) > 1:
                     raise NotImplementedError()
                 elif not var_names:
@@ -137,11 +145,11 @@ class Workflow(Base):
                     # Value is already a variable; no action.
                     continue
 
-            dir_var_defn_name = CONFIG['default_cmd_group_dir_var_name']
+            dir_var_defn_name = CONFIG.get('default_cmd_group_dir_var_name')
 
             command_groups[idx]['directory'] = '{1:}{0:}{2:}'.format(
                 dir_var_defn_name,
-                *CONFIG['variable_delimiters']
+                *CONFIG.get('variable_delimiters')
             )
 
             # Add new variable definition:
@@ -177,7 +185,7 @@ class Workflow(Base):
         for i in command_groups:
 
             dir_var_name = extract_variable_names(
-                i['directory'], CONFIG['variable_delimiters'])[0]
+                i['directory'], CONFIG.get('variable_delimiters'))[0]
 
             dir_var_defn = [i for i in self.variable_definitions
                             if i.name == dir_var_name][0]
@@ -195,6 +203,7 @@ class Workflow(Base):
             cmd_groups.append(CommandGroup(**i))
 
         self.command_groups = cmd_groups
+        self.parallel_modes = parallel_modes
 
         self.loop = loop
         for i in range(self.loop['max_iterations']):
@@ -282,6 +291,8 @@ class Workflow(Base):
         # If using an Archive with a cloud provider, check access:
         for i in archive_objs:
             if i.cloud_provider != CloudProvider.null:
+                msg = f'Checking access to cloud storage ({i.name})...'
+                print(msg, end='', flush=True)
                 i.cloud_provider.check_access()
 
     def add_submission(self, project, task_range=None):
@@ -500,12 +511,13 @@ class CommandGroup(Base):
     directory_variable_id = Column(Integer, ForeignKey('var_definition.id'))
     archive_id = Column(Integer, ForeignKey('archive.id'), nullable=True)
 
+    name = Column(String(255), nullable=True)
+    stats_name = Column(String(255), nullable=True)
     commands = Column(JSON)
     is_job_array = Column(Boolean)
     exec_order = Column(Integer)
     nesting = Column(Enum(NestingType), nullable=True)
-    modules = Column(JSON, nullable=True)
-    sources = Column(JSON, nullable=True)
+    environment = Column(JSON, nullable=True)
     _scheduler = Column('scheduler', JSON)
     profile_name = Column(String(255), nullable=True)
     profile_order = Column(Integer, nullable=True)
@@ -539,16 +551,16 @@ class CommandGroup(Base):
         return out
 
     def __init__(self, commands, directory_var, is_job_array=True,
-                 exec_order=None, nesting=None, modules=None, sources=None,
-                 scheduler=None, profile_name=None, profile_order=None, archive=None,
+                 exec_order=None, nesting=None, environment=None, scheduler=None,
+                 profile_name=None, profile_order=None, archive=None,
                  archive_excludes=None, archive_directory=None, alternate_scratch=None,
-                 stats=None):
+                 stats=None, name=None, stats_name=None):
         """Method to initialise a new CommandGroup.
 
         Parameters
         ----------
-        commands : list of str
-            List of commands to sequentially execute.
+        commands : list of dict
+            List of dicts containing commands to execute.
         directory_var : VarDefinition
             The working directory for this command group. TODO...
         is_job_array : bool, optional
@@ -570,12 +582,9 @@ class CommandGroup(Base):
             since a given task in the current command group will only begin
             once its corresponding task in the previous command group has
             completed. By default, set to `None`.
-        modules : list of str, optional
-            List of modules that are to be loaded using the `module load`
-            command, in order to ensure successful execution of the commands.
-            By default set to `None`, such that no modules are loaded.
-        sources : list of str, optional
-            List of scripts to source.
+        environment : list of str, optional
+            List of commands to be run to set up the environment for the command group. By
+            default set to `None`.
         scheduler : dict, optional
             Scheduler type and options to be passed directly to the scheduler. By default,
             `None`, in which case the DirectExecution scheduler is used and no additional
@@ -604,13 +613,14 @@ class CommandGroup(Base):
         self.is_job_array = is_job_array
         self.exec_order = exec_order
         self.nesting = nesting
-        self.modules = modules
-        self.sources = sources
+        self.environment = environment
         self.scheduler = scheduler
         self.directory_variable = directory_var
         self.profile_name = profile_name
         self.profile_order = profile_order
         self.stats = stats
+        self.name = name
+        self.stats_name = stats_name
 
         self.archive = archive
         self.archive_excludes = archive_excludes
@@ -631,12 +641,6 @@ class CommandGroup(Base):
             msg = 'At least one command must be specified.'
             raise ValueError(msg)
 
-        # Check non-empty commands exist:
-        for cmd_idx, cmd in enumerate(self.commands):
-            if not cmd:
-                msg = 'Command #{} is empty; a command must be specified.'
-                raise ValueError(msg.format(cmd_idx))
-
         self.nesting = NestingType[self.nesting] if self.nesting else None
 
         # Check alternate scratch exists
@@ -645,6 +649,17 @@ class CommandGroup(Base):
                 msg = 'Alternate scratch "{}" is not an existing directory.'
                 raise ValueError(msg.format(self.alternate_scratch))
 
+    @staticmethod
+    def get_command_lines(commands):
+        'Get all lines in the commands list.'
+        out = []
+        for i in commands:
+            if 'line' in i:
+                out.append(i['line'])
+            elif 'subshell' in i:
+                out.extend(CommandGroup.get_command_lines(i['subshell']))
+        return out
+
     @property
     def scheduler(self):
         return self._scheduler_obj
@@ -681,7 +696,9 @@ class CommandGroup(Base):
         """Get those variable names associated with this command group."""
 
         var_names = select_cmd_group_var_names(
-            self.commands, self.directory_variable.value)
+            self.get_command_lines(self.commands),
+            self.directory_variable.value
+        )
         return var_names
 
     @property
@@ -714,7 +731,7 @@ class CommandGroup(Base):
 
         cmd_group_var_defns = select_cmd_group_var_definitions(
             var_defns_dict,
-            self.commands,
+            self.get_command_lines(self.commands),
             self.directory_variable.value,
         )
 
@@ -725,26 +742,6 @@ class CommandGroup(Base):
 
         return var_defns
 
-    @property
-    def cmd_var_names(self):
-        """Get those variables definitions whose names appear directly in
-        the commands.
-
-        This excludes variable definitions that are "sub-variables".
-
-        TODO: delete this? This is the same as self.variable_names but this
-        excludes any directory variables.
-
-        """
-
-        cmd_var_names = []
-        for i in self.commands:
-            cmd_var_names.extend(
-                extract_variable_names(i, CONFIG['variable_delimiters'])
-            )
-
-        return cmd_var_names
-
 
 class VarDefinition(Base):
     """Class to represent a variable definition."""
@@ -794,7 +791,7 @@ class VarDefinition(Base):
         """Check if the variable depends on any other variables."""
 
         if extract_variable_names(self.value,
-                                  CONFIG['variable_delimiters']):
+                                  CONFIG.get('variable_delimiters')):
             return False
         else:
             return True
@@ -802,7 +799,7 @@ class VarDefinition(Base):
     def get_dependent_variable_names(self):
         """Get the names of variables on which this variable depends."""
         return extract_variable_names(self.value,
-                                      CONFIG['variable_delimiters'])
+                                      CONFIG.get('variable_delimiters'))
 
     def get_multiplicity(self, submission):
         """Get the value multiplicity of this variable for a given
@@ -1118,19 +1115,19 @@ class Submission(Base):
 
             if cg_dirs_var_vals:
                 pass
-                print(('Submission.resolve_variable_values: found existing resolved '
-                       'directory variables: {}').format(cg_dirs_var_vals), flush=True)
+                # print(('Submission.resolve_variable_values: found existing resolved '
+                #        'directory variables: {}').format(cg_dirs_var_vals), flush=True)
 
             else:
 
-                print(('Submission.resolve_variable_values: trying to resolve directory '
-                       'variable values.'), flush=True)
+                # print(('Submission.resolve_variable_values: trying to resolve directory '
+                #        'variable values.'), flush=True)
 
                 # Directory variable has not yet been resolved; try:
                 try:
                     dir_var_vals_dat = dir_var.get_values(root_directory)
-                    print(('Submission.resolve_variable_values: found directories with '
-                           'values: {}.'.format(dir_var_vals_dat)), flush=True)
+                    # print(('Submission.resolve_variable_values: found directories with '
+                    #        'values: {}.'.format(dir_var_vals_dat)), flush=True)
 
                 except UnresolvedVariableError:
                     # Move on to next command group:
@@ -1141,8 +1138,8 @@ class Submission(Base):
                     if (j not in cg_dirs_var_vals_other_val or j == '.')
                 ]
 
-                print(('Submission.resolve_variable_values: new directories are: '
-                       '{}.'.format(dir_var_vals_dat_new)), flush=True)
+                # print(('Submission.resolve_variable_values: new directories are: '
+                #        '{}.'.format(dir_var_vals_dat_new)), flush=True)
 
                 # Add VarVals:
                 for val_idx, val in enumerate(dir_var_vals_dat_new):
@@ -1158,44 +1155,44 @@ class Submission(Base):
 
             var_defns_rec = i.variable_definitions_recursive
 
-            print(('Submission.resolve_variable_values: cg_dirs_var_vals: '
-                   '{}.'.format(cg_dirs_var_vals)), flush=True)
+            # print(('Submission.resolve_variable_values: cg_dirs_var_vals: '
+            #        '{}.'.format(cg_dirs_var_vals)), flush=True)
 
             for j in cg_dirs_var_vals:
 
-                print(('Submission.resolve_variable_values: dir var val: '
-                       '{}.'.format(j)), flush=True)
+                # print(('Submission.resolve_variable_values: dir var val: '
+                #        '{}.'.format(j)), flush=True)
 
                 var_vals_dat = resolve_variable_values(
                     var_defns_rec,
                     root_directory.joinpath(j.value)
                 )
 
-                print(('Submission.resolve_variable_values: var_vals_dat: '
-                       '{}.'.format(var_vals_dat)), flush=True)
+                # print(('Submission.resolve_variable_values: var_vals_dat: '
+                #        '{}.'.format(var_vals_dat)), flush=True)
 
                 for k, v in var_vals_dat.items():
 
-                    print(('Submission.resolve_variable_values: var_vals_dat k: '
-                           '{}; v: {}.'.format(k, v)), flush=True)
+                    # print(('Submission.resolve_variable_values: var_vals_dat k: '
+                    #        '{}; v: {}.'.format(k, v)), flush=True)
 
                     vals_dat = v['vals']
                     var_defn = self.workflow.get_variable_definition_by_name(k)
 
-                    print(('Submission.resolve_variable_values: vals_dat '
-                           '{}.'.format(vals_dat)), flush=True)
-                    print(('Submission.resolve_variable_values: var_defn '
-                           '{}.'.format(var_defn)), flush=True)
+                    # print(('Submission.resolve_variable_values: vals_dat '
+                    #        '{}.'.format(vals_dat)), flush=True)
+                    # print(('Submission.resolve_variable_values: var_defn '
+                    #        '{}.'.format(var_defn)), flush=True)
 
                     if not self.is_variable_resolved(var_defn, iteration, j):
 
-                        print(('Submission.resolve_variable_values: {} not resolved...'.format(
-                            var_defn)), flush=True)
+                        # print(('Submission.resolve_variable_values: {} not resolved...'.format(
+                        #     var_defn)), flush=True)
 
                         for val_idx, val in enumerate(vals_dat):
 
-                            print(('Submission.resolve_variable_values: val: {}...'.format(
-                                val)), flush=True)
+                            # print(('Submission.resolve_variable_values: val: {}...'.format(
+                            #     val)), flush=True)
 
                             VarValue(
                                 value=val,
@@ -1258,7 +1255,7 @@ class Submission(Base):
                         all_dir_slots[k] = 'REPLACE_WITH_DIR_{}'.format(dir_idx)
 
                     wk_dirs_path = iter_path.joinpath('working_dirs_{}{}'.format(
-                        cg_sub.command_group_exec_order, CONFIG['working_dirs_file_ext']))
+                        cg_sub.command_group_exec_order, CONFIG.get('working_dirs_file_ext')))
 
                     # Make the working directory template files for each cmd group:
                     with wk_dirs_path.open('w') as handle:
@@ -1518,11 +1515,11 @@ class CommandGroupSubmission(Base):
             max_num_tasks=self.scheduler_group.get_max_num_tasks(
                 self.submission.first_iteration),
             task_step_size=cg_sub_first_iter.step_size,
-            modules=self.command_group.modules,
-            sources=self.command_group.sources,
+            environment=self.command_group.environment,
             archive=self.command_group.archive is not None,
             alternate_scratch_dir=self.alternate_scratch_dir,
-            command_group_submission_id=self.id_
+            command_group_submission_id=self.id_,
+            name=self.command_group.name,
         )
 
         js_stats_path = None
@@ -1535,6 +1532,7 @@ class CommandGroupSubmission(Base):
                     self.submission.first_iteration),
                 task_step_size=cg_sub_first_iter.step_size,
                 command_group_submission_id=self.id_,
+                name=self.command_group.stats_name,
             )
 
         out = {
@@ -1660,37 +1658,55 @@ class CommandGroupSubmission(Base):
         )
 
         for var_name, var_val_all in var_vals_normed.items():
-            var_fn = 'var_{}{}'.format(var_name, CONFIG['variable_file_ext'])
+            var_fn = 'var_{}{}'.format(var_name, CONFIG.get('variable_file_ext'))
             var_file_path = var_values_task_dir.joinpath(var_fn)
             with var_file_path.open('w') as handle:
                 for i in var_val_all:
                     handle.write('{}\n'.format(i))
 
+    @staticmethod
+    def get_formatted_commands(commands, num_cores, parallel_modes, indent=''):
+
+        # TODO: what about parallel mode env?
+        delims = CONFIG.get('variable_delimiters')
+        lns_cmd = []
+        for i in commands:
+            if 'line' in i:
+                cmd_ln = indent
+                para_mode = i.get('parallel_mode')
+                if para_mode:
+                    para_mode_config = parallel_modes.get(
+                        para_mode.lower())  # todo raise on miss
+                    para_command = para_mode_config.get('command')
+                    if para_command:
+                        cmd_ln += para_command.replace('<<num_cores>>', num_cores) + ' '
+                line = i['line']
+                for var_name in extract_variable_names(line, delims):
+                    line = line.replace(delims[0] + var_name + delims[1], f'${var_name}')
+                cmd_ln += line
+                lns_cmd.append(cmd_ln)
+            elif 'subshell' in i:
+                sub_cmds = CommandGroupSubmission.get_formatted_commands(
+                    i['subshell'],
+                    num_cores,
+                    parallel_modes,
+                    indent=(indent+'\t'),
+                )
+                lns_cmd.extend([f'{indent}('] + sub_cmds + [f'{indent})'])
+
+        return lns_cmd
+
     def write_command_file(self, project):
 
-        lns_while_start = [
-            'while true',
-            'do'
-        ]
+        lns_cmd = self.get_formatted_commands(
+            self.command_group.commands,
+            num_cores=self.command_group.scheduler.NUM_CORES_VAR,
+            parallel_modes=self.command_group.workflow.parallel_modes,
+            indent=('\t' if self.command_group.variable_definitions else ''),
+        )
 
-        delims = CONFIG['variable_delimiters']
-        lns_cmd = []
-        for i in self.command_group.commands:
-            if self.command_group.variable_definitions:
-                cmd_ln = '\t'
-            else:
-                cmd_ln = ''
-            is_parallel = 'pe' in self.command_group.scheduler.options
-            if is_parallel:
-                cmd_ln += 'mpirun -np $NSLOTS '
-            if extract_variable_names(i, delims):
-                i = i.replace(delims[0], '${').replace(delims[1], '}')
-            cmd_ln += i
-            lns_cmd.append(cmd_ln)
-
-        lns_while_end = [
-            'done \\'
-        ]
+        lns_while_start = ['while true', 'do']
+        lns_while_end = ['done \\']
 
         dt_stamp = datetime.now().strftime(r'%Y.%m.%d at %H:%M:%S')
         about_msg = ['# --- commands file generated by `hpcflow` (version: {}) '
@@ -1712,7 +1728,7 @@ class CommandGroupSubmission(Base):
 
             fd_idx = idx + 3
 
-            var_fn = 'var_{}{}'.format(i.name, CONFIG['variable_file_ext'])
+            var_fn = 'var_{}{}'.format(i.name, CONFIG.get('variable_file_ext'))
             var_file_path = ('$ITER_DIR/scheduler_group_{}/var_values'
                              '/$ZEROPAD_TASK_ID/{}').format(
                                  self.scheduler_group_index[0], var_fn)
@@ -1747,7 +1763,7 @@ class CommandGroupSubmission(Base):
         cmd_path = project.hf_dir.joinpath(
             'workflow_{}'.format(self.submission.workflow.id_),
             'submit_{}'.format(self.submission.order_id),
-            'cmd_{}{}'.format(self.command_group_exec_order, CONFIG['jobscript_ext']),
+            'cmd_{}{}'.format(self.command_group_exec_order, CONFIG.get('jobscript_ext')),
         )
         with cmd_path.open('w') as handle:
             handle.write(cmd_lns)
@@ -1757,7 +1773,7 @@ class CommandGroupSubmission(Base):
 
         # List of Paths to exclude, relative to `self.submission.workflow.directory`:
         excluded_paths = [
-            Path(CONFIG['hpcflow_directory'])] + self.submission.workflow.profile_files
+            Path(CONFIG.get('hpcflow_directory'))] + self.submission.workflow.profile_files
 
         out_dir = Path(self.command_group.scheduler.output_dir)
         err_dir = Path(self.command_group.scheduler.error_dir)
@@ -1780,10 +1796,10 @@ class CommandGroupSubmission(Base):
             'submit_{}'.format(self.submission.order_id),
             'iter_{}'.format(iteration.order_id),
             '{}_{}_{}{}'.format(
-                FILE_NAMES['alt_scratch_exc_file'],
+                CONFIG.get('alt_scratch_exc_file'),
                 self.command_group_exec_order,
                 task.order_id,
-                FILE_NAMES['alt_scratch_exc_file_ext'],
+                CONFIG.get('alt_scratch_exc_file_ext'),
             ),
         )
 
@@ -1861,12 +1877,11 @@ class CommandGroupSubmission(Base):
 
     def get_stats(self, jsonable=True, datetime_dicts=False):
         'Get task statistics for this command group submission.'
-        print('CGS.get_stats: self.command_group_submission_iterations: {}'.format(
-            self.command_group_submission_iterations))
         out = {
             'command_group_submission_id': self.id_,
             'command_group_id': self.command_group.id_,
             'commands': self.command_group.commands,
+            'name': self.command_group.name,
             'tasks': [task.get_stats(jsonable=jsonable, datetime_dicts=datetime_dicts)
                       for cgsub_iter in self.command_group_submission_iterations
                       for task in cgsub_iter.tasks
@@ -2139,7 +2154,7 @@ class Task(Base):
         """Get a list of other Tasks within the same command group that share the same
         working directory and iteration."""
         same_dir_tasks = []
-        for i in self.command_group_submission.tasks:
+        for i in self.command_group_submission_iteration.tasks:
             if i is self:
                 continue
             elif i.iteration == self.iteration:
@@ -2402,7 +2417,7 @@ class CommandGroupSubmissionIteration(Base):
             'submit_{}'.format(cg_sub.submission.order_id),
             'iter_{}'.format(self.iteration.order_id),
             'working_dirs_{}{}'.format(
-                cg_sub.command_group_exec_order, CONFIG['working_dirs_file_ext']),
+                cg_sub.command_group_exec_order, CONFIG.get('working_dirs_file_ext')),
         )
 
         with wk_dirs_path.open() as handle:
@@ -2456,10 +2471,10 @@ class SchedulerGroup(object):
         # Get num_outputs for all previous cg subs in this scheduler group
         for idx, cg_sub in enumerate(self.command_group_submissions):
 
-            print('SchedulerGroup.get_num_outputs: cg_sub idx: {}'.format(idx), flush=True)
+            # print('SchedulerGroup.get_num_outputs: cg_sub idx: {}'.format(idx), flush=True)
 
-            print('SchedulerGroup.get_num_outputs: cg_sub_iters: ')
-            pprint(cg_sub.command_group_submission_iterations)
+            # print('SchedulerGroup.get_num_outputs: cg_sub_iters: ')
+            # pprint(cg_sub.command_group_submission_iterations)
 
             cg_sub_iter = None
             for i in cg_sub.command_group_submission_iterations:
@@ -2473,9 +2488,9 @@ class SchedulerGroup(object):
             is_job_array = cg_sub.command_group.is_job_array
             nesting = cg_sub.command_group.nesting
 
-            print('SchedulerGroup.get_num_outputs: is_job_array: {}'.format(
-                is_job_array), flush=True)
-            print('SchedulerGroup.get_num_outputs: nesting: {}'.format(nesting), flush=True)
+            # print('SchedulerGroup.get_num_outputs: is_job_array: {}'.format(
+            #     is_job_array), flush=True)
+            # print('SchedulerGroup.get_num_outputs: nesting: {}'.format(nesting), flush=True)
 
             if nesting == NestingType('nest'):  # or first_cmd_group:
                 num_outs = num_outs_prev
@@ -2493,16 +2508,16 @@ class SchedulerGroup(object):
                 # cg_sub.task_multiplicity is a dict of directory keys
                 num_outs *= sum(cg_sub_iter.get_task_multiplicity().values())
 
-                print('SchedulerGroup.get_num_outputs: cg_sub_iter.task_multiplicity: {}'.format(
-                    cg_sub_iter.get_task_multiplicity()), flush=True)
+                # print('SchedulerGroup.get_num_outputs: cg_sub_iter.task_multiplicity: {}'.format(
+                #     cg_sub_iter.get_task_multiplicity()), flush=True)
 
-            print('SchedulerGroup.get_num_outputs: num_outs: {}'.format(num_outs), flush=True)
+            # print('SchedulerGroup.get_num_outputs: num_outs: {}'.format(num_outs), flush=True)
 
             num_outs_all.append(num_outs)
             num_outs_prev = num_outs
 
-        print('SchedulerGroup.get_num_outputs: num_outs_all: {}'.format(
-            num_outs_all), flush=True)
+        # print('SchedulerGroup.get_num_outputs: num_outs_all: {}'.format(
+        #     num_outs_all), flush=True)
 
         return num_outs_all
 
